PPParser subclass: #PPPattern
	instanceVariableNames: 'verificationBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPPattern commentStamp: '<historical>' prior: 0!
PPPattern is meta-parser that is solely used to match other types of parsers. It cannot be used for actually parsing something. The constructor method determines what can be matched.!


!PPPattern methodsFor: 'comparing' stamp: 'lr 4/29/2010 10:33'!
= aParser
	^ self == aParser or: [ self name notNil and: [ self name = aParser name ] ]! !

!PPPattern methodsFor: 'comparing' stamp: 'lr 4/29/2010 10:33'!
hash
	^ self identityHash! !


!PPPattern methodsFor: 'initialization' stamp: 'lr 4/29/2010 10:20'!
initializeOn: aBlock
	verificationBlock := aBlock! !


!PPPattern methodsFor: 'matching' stamp: 'lr 4/30/2010 07:53'!
copyInContext: aDictionary seen: aSeenDictionary
	^ aDictionary at: self! !

!PPPattern methodsFor: 'matching' stamp: 'lr 4/30/2010 12:01'!
match: aParser inContext: aDictionary seen: anIdentitySet
	(verificationBlock value: aParser value: aDictionary)
		ifFalse: [ ^ false ].
	^ (aDictionary at: self ifAbsentPut: [ aParser ])
		match: aParser inContext: aDictionary seen: anIdentitySet! !


!PPPattern methodsFor: 'parsing' stamp: 'lr 4/30/2010 08:48'!
parseOn: aStream
	"This is just a pattern used for matching. It should not be used in actual grammars."

	self shouldNotImplement! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPPattern class
	instanceVariableNames: ''!

!PPPattern class methodsFor: 'instance creation' stamp: 'lr 4/30/2010 08:47'!
any
	"Matches all parsers."

	^ self on: [ :parser :context | true ]! !

!PPPattern class methodsFor: 'instance creation' stamp: 'lr 4/30/2010 08:46'!
kind: aBehavior
	"Matches parsers that are of the class aBehavior."

	^ self on: [ :parser :context | parser class = aBehavior ]! !

!PPPattern class methodsFor: 'instance creation' stamp: 'lr 4/30/2010 08:46'!
name: aString
	"Matches parsers with the name aString."

	^ self on: [ :parser :context | parser name = aString ]! !

!PPPattern class methodsFor: 'instance creation' stamp: 'lr 4/29/2010 10:21'!
new
	self error: 'Use an explicit constructur on ' , self name! !

!PPPattern class methodsFor: 'instance creation' stamp: 'lr 4/30/2010 08:46'!
on: aBlock
	"Matches parsers that satisfy an arbitrary condition in aBlock."

	^ self basicNew initializeOn: aBlock! !


PPPattern subclass: #PPListPattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPListPattern commentStamp: '<historical>' prior: 0!
PPListPattern that is used to match any number of parsers. As its superclass, it cannot be used for actually parsing something.!



Object subclass: #PPProcessor
	instanceVariableNames: 'searches context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPProcessor commentStamp: '<historical>' prior: 0!
PPProcessor is an abstract superclass to PPRewriter and PPSearcher. It implements common functionality to search and transform grammars.

The implementation of these matching algorithms is inspired from the refactoring engine by Don Roberts and John Brant. Contrary to the original implementation that worked on syntax trees, this implementation was generalized and works on possibly cyclic search patterns and grammar graphs.

Instance Variables:
	searches	<Collection of: PPRule>	The rules to be processed.
	context	<Dictionary>	The current search context.!


!PPProcessor methodsFor: 'initialization' stamp: 'lr 4/29/2010 09:38'!
initialize
	super initialize.
	searches := OrderedCollection new.
	context := Dictionary new! !


!PPProcessor methodsFor: 'rules' stamp: 'lr 4/29/2010 09:34'!
addRule: aGrammarRule
	searches add: (aGrammarRule setOwner: self)! !


!PPProcessor methodsFor: 'private' stamp: 'lr 4/29/2010 09:34'!
context
	^ context! !

!PPProcessor methodsFor: 'private' stamp: 'lr 9/1/2010 20:53'!
performRule: aRule on: aParser
	context := Dictionary new.
	^ aRule performOn: aParser! !

!PPProcessor methodsFor: 'private' stamp: 'lr 9/1/2010 20:56'!
performRulesOn: aParser
	| result |
	searches do: [ :rule |
		result := self performRule: rule on: aParser.
		result notNil ifTrue: [ ^ result ] ].
	^ nil! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPProcessor class
	instanceVariableNames: ''!

!PPProcessor class methodsFor: 'instance creation' stamp: 'lr 4/30/2010 08:34'!
new
	^ self basicNew initialize! !


Object subclass: #PPRule
	instanceVariableNames: 'owner search'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPRule commentStamp: '<historical>' prior: 0!
PPRule is the abstract superclass of all of the grammar search rules. A rule is the first class representation of a particular pattern to search for. The owner of the rule is the algorithms that actually executes the search. This arrangement allows multiple searches to be conducted by a single processor.

Instance Variables:
	owner	<PPProcessor>	The processor that is actually performing the search.
	search	<PPParser>	The parse pattern to be searched.
!


!PPRule methodsFor: 'initialization' stamp: 'lr 4/29/2010 08:51'!
initialize! !

!PPRule methodsFor: 'initialization' stamp: 'lr 4/28/2010 20:45'!
setOwner: aGrammarSearcher
	owner := aGrammarSearcher! !

!PPRule methodsFor: 'initialization' stamp: 'lr 4/29/2010 08:23'!
setSearch: aParser
	search := aParser! !


!PPRule methodsFor: 'matching' stamp: 'lr 4/28/2010 21:10'!
canMatch: aParser 
	^ true! !

!PPRule methodsFor: 'matching' stamp: 'lr 4/29/2010 08:14'!
foundMatchFor: aParser 
	self subclassResponsibility! !

!PPRule methodsFor: 'matching' stamp: 'lr 4/29/2010 08:53'!
performOn: aParser
	(search match: aParser inContext: owner context)
		ifFalse: [ ^ nil ].
	(self canMatch: aParser)
		ifFalse: [ ^ nil ].
	^ self foundMatchFor: aParser! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPRule class
	instanceVariableNames: ''!

!PPRule class methodsFor: 'instance creation' stamp: 'lr 4/29/2010 09:51'!
new
	^ self basicNew initialize! !

!PPRule class methodsFor: 'instance creation' stamp: 'lr 4/29/2010 09:51'!
searchFor: aParser
	^ self new setSearch: aParser! !


PPRule subclass: #PPSearchRule
	instanceVariableNames: 'answerBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPSearchRule commentStamp: '<historical>' prior: 0!
PPSearchRule is a rule that simply searches for matches to the rule. Every time a match is found, answerBlock is evaluated with the parser that matches and the current answer. This two-argument approach allows a collection to be formed from all of the matches, like with #inject:into:.

Instance Variables:
	answerBlock	<BlockClosure>	Block to evaluate with the matching node and the current answer.
!


!PPSearchRule methodsFor: 'initialization' stamp: 'lr 4/29/2010 09:51'!
setAnswerBlock: aBlock
	answerBlock := aBlock! !


!PPSearchRule methodsFor: 'matching' stamp: 'lr 6/5/2011 16:51'!
canMatch: aParser
	owner setAnswer: (answerBlock cull: aParser cull: owner answer).
	^ super canMatch: aParser! !

!PPSearchRule methodsFor: 'matching' stamp: 'lr 4/29/2010 08:15'!
foundMatchFor: aParser 
	^ aParser! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPSearchRule class
	instanceVariableNames: ''!

!PPSearchRule class methodsFor: 'instance creation' stamp: 'lr 4/29/2010 08:21'!
searchFor: aParser thenDo: aBlock
	^ (self searchFor: aParser) setAnswerBlock: aBlock! !


PPRule subclass: #PPReplaceRule
	instanceVariableNames: 'verificationBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPReplaceRule commentStamp: '<historical>' prior: 0!
PPReplaceRule is the abstract superclass of all of the transforming rules. The rules change the grammar by replacing the node that matches the rule. Subclasses implement different strategies for this replacement.

Instance Variables:
	verificationBlock	<BlockClosure>	Is evaluated with the matching parser and allows for further verification of a match.!


!PPReplaceRule methodsFor: 'initialization' stamp: 'lr 4/29/2010 08:13'!
initialize
	super initialize.
	verificationBlock := [ :parser | true ]! !

!PPReplaceRule methodsFor: 'initialization' stamp: 'lr 4/29/2010 08:26'!
setVerificationBlock: aBlock
	verificationBlock := aBlock! !


!PPReplaceRule methodsFor: 'matching' stamp: 'lr 6/5/2011 16:52'!
canMatch: aParser 
	^ verificationBlock cull: aParser! !


PPReplaceRule subclass: #PPParserReplaceRule
	instanceVariableNames: 'replaceParser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPParserReplaceRule commentStamp: '<historical>' prior: 0!
PPParserReplaceRule replaces a matched grammar with another grammar, which may include patterns from the matching grammar.

Instance Variables:
	replaceParser	<PPParser>	The parser to replace the matched parser with.!


!PPParserReplaceRule methodsFor: 'initialization' stamp: 'lr 4/29/2010 08:29'!
setReplaceParser: aParser
	replaceParser := aParser! !


!PPParserReplaceRule methodsFor: 'matching' stamp: 'lr 4/29/2010 08:16'!
foundMatchFor: aParser 
	^ replaceParser copyInContext: owner context! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPParserReplaceRule class
	instanceVariableNames: ''!

!PPParserReplaceRule class methodsFor: 'instance creation' stamp: 'lr 4/29/2010 08:28'!
searchFor: aSearchParser replaceWith: aReplaceParser when: aVerificationBlock
	^ (self searchFor: aSearchParser) 
		setReplaceParser: aReplaceParser;
		setVerificationBlock: aVerificationBlock;
		yourself! !


PPProcessor subclass: #PPSearcher
	instanceVariableNames: 'answer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPSearcher commentStamp: '<historical>' prior: 0!
PPSearcher walks over a grammar specification and matches its parsers against the patterns using #match:inContext:.

Instance Variables:
	answer <Object>	The answer propagated between matches.!


!PPSearcher methodsFor: 'initialization' stamp: 'lr 4/29/2010 09:37'!
setAnswer: anObject
	answer := anObject! !


!PPSearcher methodsFor: 'public' stamp: 'lr 4/29/2010 09:45'!
execute: aParser
	"Perform the search rules of the receiver on aParser. Answer the result of the search."

	^ self execute: aParser initialAnswer: nil! !

!PPSearcher methodsFor: 'public' stamp: 'lr 9/1/2010 20:56'!
execute: aParser initialAnswer: anObject
	"Perform the search rules of the receiver on aParser. Inject anObject into the matches and answer the result."

	| previous |
	previous := context.
	answer := anObject.
	context := Dictionary new.
	aParser allParsersDo: [ :each | 
		self performRulesOn: each ].
	context := previous.
	^ answer! !


!PPSearcher methodsFor: 'rules' stamp: 'lr 4/29/2010 09:48'!
matches: aParser do: anAnswerBlock
	"Add a search expression aParser, evaluate anAnswerBlock with the matched node and the previous answer."

	self addRule: (PPSearchRule searchFor: aParser thenDo: anAnswerBlock)! !

!PPSearcher methodsFor: 'rules' stamp: 'lr 4/29/2010 09:56'!
matchesAnyOf: aCollectionOfParsers do: anAnswerBlock
	"Add a collection of search expressions aCollectionOfParsers, evaluate anAnswerBlock with the matched node and the previous answer."
	
	aCollectionOfParsers
		do: [ :each | self matches: each do: anAnswerBlock ]! !


!PPSearcher methodsFor: 'private' stamp: 'lr 4/29/2010 09:46'!
answer
	^ answer! !


PPReplaceRule subclass: #PPBlockReplaceRule
	instanceVariableNames: 'replaceBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPBlockReplaceRule commentStamp: '<historical>' prior: 0!
PPBlockReplaceRule replaces the matching node by the result of evaluating replaceBlock. This allows arbitrary computation to come up with a replacement.

Instance Variables:
	replaceBlock <BlockClosure>	The block that returns the parer to replace to matching parser with.
!


!PPBlockReplaceRule methodsFor: 'initialization' stamp: 'lr 4/29/2010 08:29'!
setReplaceBlock: aBlock
	replaceBlock := aBlock! !


!PPBlockReplaceRule methodsFor: 'matching' stamp: 'lr 6/5/2011 16:51'!
foundMatchFor: aParser 
	^ replaceBlock cull: aParser! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPBlockReplaceRule class
	instanceVariableNames: ''!

!PPBlockReplaceRule class methodsFor: 'instance creation' stamp: 'lr 4/29/2010 08:28'!
searchFor: aSearchParser replaceWith: aReplaceBlock when: aVerificationBlock
	^ (self searchFor: aSearchParser) 
		setReplaceBlock: aReplaceBlock;
		setVerificationBlock: aVerificationBlock;
		yourself! !


PPProcessor subclass: #PPRewriter
	instanceVariableNames: 'changed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!
!PPRewriter commentStamp: '<historical>' prior: 0!
PPRewriter walks over a grammar graph and transforms its parsers. If the grammar is modified, #hasChanged returns true.

Instance Variables:
	changed	<Boolean>	Indicates if the last operation has changed anything.!


!PPRewriter methodsFor: 'accessing' stamp: 'lr 4/29/2010 10:16'!
replace: aSearchParser with: aReplaceParser
	self replace: aSearchParser with: aReplaceParser when: [ :node | true ]! !

!PPRewriter methodsFor: 'accessing' stamp: 'lr 4/29/2010 08:25'!
replace: aSearchParser with: aReplaceParser when: aValidationBlock
	self addRule: (PPParserReplaceRule searchFor: aSearchParser replaceWith: aReplaceParser when: aValidationBlock)! !

!PPRewriter methodsFor: 'accessing' stamp: 'lr 4/29/2010 10:16'!
replace: aSearchParser withValueFrom: aReplaceBlock
	self replace: aSearchParser withValueFrom: aReplaceBlock when: [ :node | true ]! !

!PPRewriter methodsFor: 'accessing' stamp: 'lr 4/29/2010 08:25'!
replace: aSearchParser withValueFrom: aReplaceBlock when: aValidationBlock
	self addRule: (PPBlockReplaceRule searchFor: aSearchParser replaceWith: aReplaceBlock when: aValidationBlock)! !


!PPRewriter methodsFor: 'initialization' stamp: 'lr 4/29/2010 21:28'!
initialize
	super initialize.
	changed := false! !


!PPRewriter methodsFor: 'public' stamp: 'lr 9/1/2010 20:58'!
execute: aParser
	"Perform the replace rules of the receiver on aParser, answer the resulting parser."

	| previous result |
	previous := context.
	changed := false.
	context := Dictionary new.
	result := aParser transform: [ :each |
		| transformed |
		transformed := self performRulesOn: each.
		transformed isNil
			ifTrue: [ each ]
			ifFalse: [ changed := true. transformed ] ].
	context := previous.
	^ result! !


!PPRewriter methodsFor: 'testing' stamp: 'lr 4/29/2010 21:28'!
hasChanged
	"Answer if the last operation has changed anything."
	
	^ changed! !


PPEpsilonParser subclass: #PPSentinel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitAnalyzer-Core'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPSentinel class
	instanceVariableNames: 'current instance'!

!PPSentinel class methodsFor: 'instance creation' stamp: 'lr 9/16/2010 17:54'!
instance
	^ instance ifNil: [ instance := self new ]! !
