﻿ReadStream subclass: #PPStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!
!PPStream commentStamp: '<historical>' prior: 0!
A positional stream implementation used for parsing. It overrides some methods for optimization reasons.!


!PPStream methodsFor: 'accessing' stamp: 'lr 4/29/2008 21:48'!
peek
	"An improved version of peek, that is slightly faster than the built in version."

	^ self atEnd ifFalse: [ collection at: position + 1 ]! !

!PPStream methodsFor: 'accessing' stamp: 'lr 8/25/2010 11:36'!
position: anInteger
	"The receiver does not check for invalid arguments passed to this method, as it is solely used with valid indexes for backtracking."

	position := anInteger! !

!PPStream methodsFor: 'accessing' stamp: 'lr 10/5/2010 16:29'!
uncheckedPeek
	"An unchecked version of peek that throws an error if we try to peek over the end of the stream, even faster than #peek."

	^ collection at: position + 1! !


!PPStream methodsFor: 'converting' stamp: 'lr 2/7/2010 20:53'!
asPetitStream
	^ self! !


!PPStream methodsFor: 'printing' stamp: 'lr 11/4/2010 19:23'!
printOn: aStream
	collection isString
		ifFalse: [ ^ super printOn: aStream ].
	aStream
		nextPutAll: (collection copyFrom: 1 to: position);
		nextPutAll: '·';
		nextPutAll: (collection copyFrom: position + 1 to: readLimit)! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPStream class
	instanceVariableNames: ''!


ReadStream subclass: #PPStreamer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!


ReadStream subclass: #PPScreamer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPScreamer methodsFor: 'as yet unclassified' stamp: 'JorgeRessia 12/13/2011 15:41'!
asPetitNumber

| aParser aParseObject stream result aTargetObject anInteger |
	self 
		assert: aParser isPetitParser
		description: 'Parser invalid'.
	stream := aParseObject asPetitStream.
	result := aParser parse: stream.
	aTargetObject isNil
		ifTrue: [ self deny: result isPetitFailure ]
		ifFalse: [ self assert: result = aTargetObject ].
	self 
		assert: stream position = anInteger
		description: 'Parser accepted at wrong position'.
	self 
		assert: aParser isPetitParser
		description: 'Parser invalid'.
	stream := aParseObject asPetitStream.
	result := aParser parse: stream.
	aTargetObject isNil
		ifTrue: [ self deny: result isPetitFailure ]
		ifFalse: [ self assert: result = aTargetObject ].

	^ self asScreamer! !

!PPScreamer methodsFor: 'as yet unclassified' stamp: 'JorgeRessia 12/13/2011 15:41'!
asScreamer

| aParser aParseObject stream result aTargetObject anInteger |
	self 
		assert: aParser isPetitParser
		description: 'Parser invalid'.
	stream := aParseObject asPetitStream.
	result := aParser parse: stream.
	aTargetObject isNil
		ifTrue: [ self deny: result isPetitFailure ]
		ifFalse: [ self assert: result = aTargetObject ].
	self 
		assert: stream position = anInteger
		description: 'Parser accepted at wrong position'.
	self 
		assert: aParser isPetitParser
		description: 'Parser invalid'.
	stream := aParseObject asPetitStream.
	result := aParser parse: stream.
	aTargetObject isNil
		ifTrue: [ self deny: result isPetitFailure ]
		ifFalse: [ self assert: result = aTargetObject ].
	^ self asShout! !

!PPScreamer methodsFor: 'as yet unclassified' stamp: 'JorgeRessia 12/13/2011 15:39'!
asShout
	^ self asPetitNumber! !


Object subclass: #PPTokenizer
	instanceVariableNames: 'collection start stop'
	classVariableNames: 'NewLineParser'
	poolDictionaries: ''
	category: 'PetitParser-Core'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPTokenizer class
	instanceVariableNames: ''!

!PPTokenizer class methodsFor: 'as yet unclassified' stamp: 'JorgeRessia 12/13/2011 15:33'!
new
	^ self basicNew ! !


Object subclass: #PPToken
	instanceVariableNames: 'collection start stop'
	classVariableNames: 'NewLineParser'
	poolDictionaries: ''
	category: 'PetitParser-Core'!
!PPToken commentStamp: '<historical>' prior: 0!
PPToken represents a parsed part of the input stream. Contrary to a simple String it remembers where it came from, the original collection and its start and stop position.

Instance Variables:
	collection	<SequenceableCollection>	The collection this token comes from.
	start	<Integer>	The start position in the collection.
	stop	<Integer>	The stop position in the collection.!


!PPToken methodsFor: 'accessing' stamp: 'lr 6/15/2010 23:34'!
collection
	"Answer the underlying collection of this token."

	^ collection! !

!PPToken methodsFor: 'accessing' stamp: 'lr 6/15/2010 23:39'!
size
	"Answer the size of this token."

	^ stop - start + 1! !

!PPToken methodsFor: 'accessing' stamp: 'lr 6/15/2010 23:33'!
start
	"Answer the start position of this token in the underlying collection."

	^ start! !

!PPToken methodsFor: 'accessing' stamp: 'lr 6/15/2010 23:33'!
stop
	"Answer the stop position of this token in the underlying collection."
	
	^ stop! !

!PPToken methodsFor: 'accessing' stamp: 'lr 6/15/2010 23:34'!
value
	"Answer the contents of this token."

	^ collection copyFrom: start to: stop! !


!PPToken methodsFor: 'comparing' stamp: 'lr 10/7/2009 09:06'!
= anObject
	^ self class = anObject class and: [ self value = anObject value ]! !

!PPToken methodsFor: 'comparing' stamp: 'lr 10/7/2009 09:06'!
hash
	^ self value hash! !


!PPToken methodsFor: 'copying' stamp: 'lr 6/16/2008 10:55'!
copyFrom: aStartInteger to: aStopInteger
	^ self class on: collection start: start + aStartInteger - 1 stop: stop + aStopInteger - 3! !


!PPToken methodsFor: 'initialization' stamp: 'lr 4/30/2010 12:13'!
initializeOn: aSequenceableCollection start: aStartInteger stop: aStopInteger
	collection := aSequenceableCollection.
	start := aStartInteger.
	stop := aStopInteger! !


!PPToken methodsFor: 'printing' stamp: 'lr 6/16/2008 10:13'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self value; nextPut: $)! !


!PPToken methodsFor: 'querying' stamp: 'lr 9/7/2011 20:40'!
column
	"Answer the column number of this token in the underlying collection."
	
	| position |
	position := 0.
	(NewLineParser , [ :stream |
		start <= stream position
			ifTrue: [ ^ start - position ].
		position := stream position ] asParser
		/ #any asParser) star
			parse: collection.
	 ^ start - position! !

!PPToken methodsFor: 'querying' stamp: 'lr 9/7/2011 20:41'!
line
	"Answer the line number of this token in the underlying collection."
	
	| line |
	line := 1.
	(NewLineParser , [ :stream |
		start <= stream position
			ifTrue: [ ^ line ].
		line := line + 1 ] asParser
		/ #any asParser) star
			parse: collection.
	^ line! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPToken class
	instanceVariableNames: ''!

!PPToken class methodsFor: 'initialization' stamp: 'lr 9/7/2011 20:40'!
initialize
	"Platform independent newline sequence. LF: Unix, CR+LF: Windows, and CR: Apple."
	NewLineParser := (Character lf asParser) / (Character cr asParser , Character lf asParser optional)! !


!PPToken class methodsFor: 'instance creation' stamp: 'lr 4/6/2010 20:58'!
new
	self error: 'Token can only be created using a dedicated constructor.'! !

!PPToken class methodsFor: 'instance creation' stamp: 'lr 4/30/2010 12:13'!
on: aSequenceableCollection
	^ self on: aSequenceableCollection start: 1 stop: aSequenceableCollection size! !

!PPToken class methodsFor: 'instance creation' stamp: 'JorgeRessia 12/13/2011 15:32'!
on: aSequenceableCollection start: aStartInteger stop: aStopInteger
	(Smalltalk at: #PPTokenizer) new.
	^ self basicNew 
		initializeOn: aSequenceableCollection
		start: aStartInteger stop: aStopInteger! !


Object subclass: #PPFailure
	instanceVariableNames: 'message position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!
!PPFailure commentStamp: '<historical>' prior: 0!
The failure object in PetitParser. It is the only class that responds to #isPetitFailure with true. It contains an error message and a position of the occurrence of the failure.

Instance Variables:
	message	<String>	The error message of this failure.
	position	<Integer>	The position of this failure in the input stream.
!


!PPFailure methodsFor: 'accessing' stamp: 'lr 5/5/2010 13:56'!
message
	"Answer a human readable error message of this parse failure."
	
	^ message! !

!PPFailure methodsFor: 'accessing' stamp: 'lr 5/5/2010 13:55'!
position
	"Answer the position in the source string that caused this parse failure."

	^ position! !


!PPFailure methodsFor: 'initialization' stamp: 'lr 5/5/2010 13:55'!
initializeMessage: aString at: anInteger
	message := aString.
	position := anInteger! !


!PPFailure methodsFor: 'printing' stamp: 'lr 5/5/2010 14:01'!
printOn: aStream
	aStream nextPutAll: self message; nextPutAll: ' at '; print: position! !


!PPFailure methodsFor: 'testing' stamp: 'lr 2/7/2010 20:54'!
isPetitFailure
	"I am the only class that should implement this method to return true."

	^ true! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPFailure class
	instanceVariableNames: ''!

!PPFailure class methodsFor: 'instance creation' stamp: 'lr 5/5/2010 13:56'!
message: aString at: anInteger
	^ self basicNew initializeMessage: aString at: anInteger! !


Object subclass: #PPMemento
	instanceVariableNames: 'result count position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!
!PPMemento commentStamp: '<historical>' prior: 0!
PPMemento is an internal class used by PPMemoizedParser to cache results and detect left-recursive calls.

Instance Variables:
	result	<Object>	The cached result.
	count	<Integer>	The number of recursive cycles followed.
	position	<Integer>	The position of the cached result in the input stream.!


!PPMemento methodsFor: 'accessing' stamp: 'lr 4/22/2008 18:23'!
position
	^ position! !

!PPMemento methodsFor: 'accessing' stamp: 'lr 4/26/2008 15:48'!
position: anInteger
	position := anInteger! !

!PPMemento methodsFor: 'accessing' stamp: 'lr 4/24/2008 10:15'!
result
	^ result! !

!PPMemento methodsFor: 'accessing' stamp: 'lr 4/22/2008 18:23'!
result: anObject
	result := anObject! !


!PPMemento methodsFor: 'accessing-readonly' stamp: 'lr 4/22/2008 18:23'!
count
	^ count! !


!PPMemento methodsFor: 'actions' stamp: 'lr 4/22/2008 18:20'!
increment
	count := count + 1! !


!PPMemento methodsFor: 'initialization' stamp: 'lr 4/22/2008 18:21'!
initialize
	count := 0
	! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PPMemento class
	instanceVariableNames: ''!

!PPMemento class methodsFor: 'instance creation' stamp: 'lr 4/22/2008 18:21'!
new
	^ self basicNew initialize! !


Object subclass: #PPMemory
	instanceVariableNames: 'result count position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPMemory methodsFor: 'as yet unclassified' stamp: 'JorgeRessia 12/12/2011 21:25'!
end
	^ self! !

!PPMemory methodsFor: 'as yet unclassified' stamp: 'JorgeRessia 12/12/2011 21:25'!
start
	^ self! !


Object subclass: #PPError
	instanceVariableNames: 'message position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPError methodsFor: 'as yet unclassified' stamp: 'JorgeRessia 12/12/2011 21:31'!
fail
	^ self ! !

PPToken initialize!